require 'rest-client'
require 'json'
require 'logger'

module WGU
  module PPSCommons
    def self.included(base)
      base.send :include, InstanceMethods
    end

    # search through a hash's nested keys for crap
    def PPSCommons.deep_find(key, object=self, found=nil)
      if object.respond_to?(:key?) && object.key?(key)
        object[key]
      elsif object.respond_to?(:each)
        object.find { |k,v| found = deep_find(key, v) }
        found
      end
    end

    # this tells EventMachine how many threads it can use.  There is a lot of
    # Deferrals in this project so it helps to run a higher number.
    # EM defaults to '20', PPS defaults to '40'.  This can be adjusted by
    # adjusting the `PPS_THREAD_COUNT` environment variable
    def PPSCommons.unleash_the_fury_level
      found = ENV['PPS_THREAD_COUNT'].to_i
      found.eql?(0) ? 40 : found.to_i
    end

    # Common instance methods
    module InstanceMethods
      def logger(file = ENV['PPS_COMMONS_DEFAULT_LOG_LOCATION'] || STDOUT)
        if @logger.nil?
          log = file.eql?(STDOUT) ? file : nil

          unless file.eql? STDOUT
            log = ::File.expand_path(file)
            unless ::File.exist?(log)
              log_dir = File.dirname(log)
              unless ::Dir.exist?(log_dir)
                ::FileUtils.mkdir_p(log_dir)
              end
              ::File.new(log, 'a+')
            end
          end

          Logger.new(log)
        else
          @logger
        end
      end
      # instance version of +self.deep_find()+
      def deep_find(key, object=self, found=nil)
        WGU::PPSCommons.deep_find(key, object)
      end
    end
  end
end
